@@ -2,11 +2,14 @@ import asyncio
 import logging
 
 from time import time
+
 from .. import bhf_signals
 from contextlib import suppress
 from inspect import isawaitable
 from ipaddress import ip_address
-from typing import Awaitable, Callable, Dict, Generator, Optional, Tuple, Type, TypeVar, Union
+from bhfutils.crawler.playwright.stealth import stealth_async
+from bhfutils.crawler.playwright.cursor.do_async import GhostCursor, create_cursor
+from typing import Union, Awaitable, Literal, Callable, Dict, Generator, Optional, Tuple, Type, TypeVar
 
 from playwright.async_api import (
     BrowserContext,
@@ -16,7 +19,6 @@ from playwright.async_api import (
     Response as PlaywrightResponse,
     Route,
 )
-from playwright_stealth import stealth_async
 from scrapy import Spider, signals
 from scrapy.crawler import Crawler
 from scrapy.http import Request, Response
@@ -31,7 +33,7 @@ from scrapy.core.downloader.handlers.http import HTTPDownloadHandler
 from w3lib.encoding import html_body_declared_encoding, http_content_type_encoding
 
 from .headers import use_scrapy_headers
-from .page import PageMethod
+from .page import CursorMethod, PageMethod
 
 __all__ = ["ScrapyPlaywrightDownloadHandler"]
 
@@ -60,6 +62,16 @@ def _make_response_logger(context_name: str) -> Callable:
     return _log_request
 
 
+def _get_cursor_method_result(cursor: GhostCursor, cm: CursorMethod):
+    try:
+        method = getattr(cursor, cm.method)
+    except AttributeError:
+        logger.warning(f"Ignoring {repr(cm)}: could not find method")
+        return None
+    else:
+        return method(*cm.args, **cm.kwargs)
+
+
 def _get_page_method_result(page: Page, pm: PageMethod):
     try:
         method = getattr(page, pm.method)
@@ -117,7 +129,8 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
             self.abort_request = load_object(settings["PLAYWRIGHT_ABORT_REQUEST"])
 
         self.user_agent_rotation_idx = 0
-        self.page_load_until: str = settings.get("PLAYWRIGHT_PAGE_LOAD_UNTIL") or "load"
+        self.page_load_until: Literal["commit", "domcontentloaded", "load", "networkidle"] = settings.get(
+            "PLAYWRIGHT_PAGE_LOAD_UNTIL") or "load"
         self.playwright_user_agents: list = crawler.settings.get("PLAYWRIGHT_USER_AGENTS") or []
         crawler.signals.connect(self._ip_changed, bhf_signals.ip_changed)
 
@@ -127,7 +140,10 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
 
     def _engine_started(self) -> Deferred:
         """Launch the browser. Use the engine_started signal as it supports returning deferreds."""
-        return deferred_from_coro(self._launch_browser())
+        if not self.playwright_user_agents:
+            return deferred_from_coro(self._launch_browser())
+        else:
+            return deferred_from_coro(self._launch_browser(user_agent=self.playwright_user_agents[1]))
 
     def _ip_changed(self) -> Deferred:
         """Plan User-Agent rotation"""
@@ -182,7 +198,7 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
         context = self.contexts.get("default")
 
         page = await context.new_page()
-        await stealth_async(page)
+        # await stealth_async(page)
         self.stats.inc_value("playwright/page_count")
         logger.debug(
             "[Context=%s] New page created, page count is %i (%i for all contexts)",
@@ -247,7 +263,7 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
                             f"Spider '{spider.name}' does not have a '{handler}' attribute,"
                             f" ignoring handler for event '{event}'"
                         )
-
+            # overwrite request handler
             await page.unroute("**")
             await page.route(
                 "**",
@@ -350,8 +366,13 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
                 if result is not None:
                     pm.result = await result if isawaitable(result) else result
                     await page.wait_for_load_state(timeout=self.default_navigation_timeout)
+            elif isinstance(pm, CursorMethod):
+                cursor = create_cursor(page)
+                result = _get_cursor_method_result(cursor, pm)
+                if result is not None:
+                    pm.result = await result if isawaitable(result) else result
             else:
-                logger.warning(f"Ignoring {repr(pm)}: expected PageMethod, got {repr(type(pm))}")
+                logger.warning(f"Ignoring {repr(pm)}: expected PageMethod or CursorMethod, got {repr(type(pm))}")
 
     def _increment_request_stats(self, request: PlaywrightRequest) -> None:
         stats_prefix = "playwright/request_count"
