@@ -2,7 +2,25 @@ import warnings
 
 from scrapy.exceptions import ScrapyDeprecationWarning
 
-__all__ = ["PageMethod"]
+__all__ = ["CursorMethod", "PageMethod"]
+
+
+class CursorMethod:
+    """
+    Represents a method to be called (and awaited if necessary) on a
+    Playwright page cursor, such as "click", "move", "move_to", etc.
+    """
+
+    def __init__(self, method: str, *args, **kwargs) -> None:
+        self.method = method
+        self.args = args
+        self.kwargs = kwargs
+        self.result = None
+
+    def __str__(self):
+        return f"<{self.__class__.__name__} for method '{self.method}'>"
+
+    __repr__ = __str__
 
 
 class PageMethod:
